type terme =
    V of string
  | C of string
  | U of string * terme
  | B of string * terme * terme

exception Erreur

type 'a algbre = {
  eval_c : string -> 'a;
  eval_u : string -> 'a -> 'a;
  eval_b : string -> 'a -> 'a -> 'a;
} 

val value : 'a algbre -> terme -> (string * 'a) list -> 'a
type qualit =
    Constante
  | Variable
  | Prfixe
  | Postfixe
  | Gassoc
  | Dassoc
  | Nassoc

val imprime_inf : (string -> int) -> (int -> qualit) -> terme -> unit

val analyse_inf :
  int -> (string -> int) -> (int -> qualit) -> string list -> terme

type er = terme

val er_print : er -> unit
val er_of_string : string -> er

type 'a t = ('a -> 'a -> int) * 'a list
val empty : ('a -> 'a -> int) -> 'a t
val is_empty : 'a t -> bool
val mem : 'a -> 'a t -> bool
val add : 'a -> 'a t -> 'a t
val remove : 'a -> 'a t -> 'a t
val union : 'a t -> 'a t -> 'a t
val inter : 'a t -> 'a t -> 'a t
val diff : 'a t -> 'a t -> 'a t
val compare : 'a t -> 'a t -> int
val elements : 'a t -> 'a list
val iter : ('a -> unit) -> 'a t -> unit
val equal : 'a t -> 'a t -> bool

type tat = { mutable transitions : (char * tat) list; numro : int; } 
type af = { initial : tat t; final : tat t; contient_mu : bool; } 
val af_print : af -> unit
val affiche_af :
  ?ce:string ->
  ?d:float ->
  ?lf:float ->
  ?eh:float ->
  ?ehp:float ->
  ?ehpp:float ->
  ?lb:float ->
  ?mh:float ->
  ?ct:string ->
  ?ra:float -> 
  ?ev:float -> 
  ?dv:float -> 
  ?hv:float -> 
  af -> unit
val compteur_d'tats : int ref
val vide : tat t
val af_delta : tat t -> char -> tat t
val af_rec : af -> string -> bool
val nouvel_tat : unit -> tat
val ajoute_transition : tat -> char -> tat -> unit
val af_dfinir : tat list -> tat list -> af
val af_vide : af
val af_motvide : unit -> af
val af_car : char -> af
val af_add : af -> af -> af
val af_mult : af -> af -> af
val af_toile : af -> af
val af_of_er : er -> af

val dfinir_alphabet : string -> unit
val nombre_de_lettres : unit -> int
val lettre_of_numro : int -> char
val numro_of_lettre : char -> int

type afd = { delta : int array array; accept : bool array; } 
val afd_print : afd -> unit
val affiche_afd :
  ?ce:string ->
  ?d:float ->
  ?lf:float ->
  ?eh:float ->
  ?ehp:float ->
  ?ehpp:float ->
  ?lb:float ->
  ?mh:float ->
  ?ct:string ->
  ?ra:float -> 
  ?ev:float -> 
  ?dv:float -> 
  ?hv:float -> 
  afd -> unit
val afd_ntats : afd -> int
val afd_rec : afd -> string -> bool
val construire_afd :
  ('a -> 'a -> bool) -> 'a -> ('a -> char -> 'a) -> ('a -> bool) -> afd
val afd_of_af : af -> afd
val af_of_afd : afd -> af

val afd_complmentaire : afd -> unit
val afd_union : afd -> afd -> afd
val afd_inter : afd -> afd -> afd
val afd_diff : afd -> afd -> afd
val afd_diffsym : afd -> afd -> afd
val afd_vide : afd -> bool
val afd_inclus : afd -> afd -> bool
val afd_gal : afd -> afd -> bool

val am_of_afd : afd -> afd

val af_of_string : string -> af
val afd_of_string : string -> afd
val am_of_string : string -> afd
val afd_of_er : er -> afd
val am_of_er : er -> afd
val am_of_af : af -> afd

val fichier_courant : out_channel ref
val open_file : string -> unit
val close_file : unit -> unit
val output_string : string -> unit
val output_int : int -> unit
val output_afd : char -> afd -> unit
val output_fichier_afd : string -> afd -> unit

